pub enum LambdaType {
A,
E,
T,
Composition(Box<LambdaType>, Box<LambdaType>),
}Expand description
The basic type system of the lambda calculus and LOT.
Variants
A
A type for crate::Actors
E
A type for crate::Events
T
A type for truth values.
Composition(Box<LambdaType>, Box<LambdaType>)
A type for functions
Implementations
Sourceimpl LambdaType
impl LambdaType
Sourcepub fn lift_type(self) -> LambdaType
pub fn lift_type(self) -> LambdaType
Takes a type x and returns <<x,t>, t>
Sourcepub fn retrievable_types(&self) -> RetrievableTypeIterator<'_> ⓘ
pub fn retrievable_types(&self) -> RetrievableTypeIterator<'_> ⓘ
Get all types which can be created from this type (what you would need to pass before to produce that type).
Examples
- <a,t> can create t (arg: a)
- <a,<a,t>> can create t: (arg: a) or <a,t> (arg: a)
- <a, <<a,t>, <t,t>>> can create <<a,t>, <t,t>> (arg: a), <t,t> (arg: <a,t>) or t
let x = LambdaType::from_string("<a, <<a,t>, <t,t>>>")?
.retrievable_types()
.map(|(_, x)| x.to_string())
.collect::<Vec<_>>();
assert_eq!(x, vec!["<<a,t>,<t,t>>", "<t,t>", "t"]);Sourcepub fn is_lifted_type_of(&self, t: &LambdaType) -> bool
pub fn is_lifted_type_of(&self, t: &LambdaType) -> bool
Checks if the type is the lifted version of another.
Sourcepub fn add_right_argument(&mut self, other: LambdaType)
pub fn add_right_argument(&mut self, other: LambdaType)
Takes a type t and argument x and returns <t, x>
Sourcepub fn add_left_argument(&mut self, other: LambdaType)
pub fn add_left_argument(&mut self, other: LambdaType)
Takes a type t and argument x and returns <x, t>
Sourcepub fn from_string(s: &str) -> Result<Self, TypeParsingError>
pub fn from_string(s: &str) -> Result<Self, TypeParsingError>
Parse a type
let x = LambdaType::from_string("<e,<e,t>>")?;Errors
Returns a TypeParsingError if the string is malformed and doesn’t represent a type.
Sourcepub fn split(&self) -> Result<(&LambdaType, &LambdaType), TypeError>
pub fn split(&self) -> Result<(&LambdaType, &LambdaType), TypeError>
Sourcepub fn is_function(&self) -> bool
pub fn is_function(&self) -> bool
Checks that the type is a function.
Sourcepub fn is_one_place_function(&self) -> bool
pub fn is_one_place_function(&self) -> bool
Checks if the type takes a primitive type and returns a primitive type
Sourceimpl LambdaType
impl LambdaType
Sourcepub fn random_no_e(r: &mut impl Rng) -> Self
pub fn random_no_e(r: &mut impl Rng) -> Self
Returns a random type, except for LambdaType::E